#!/usr/bin/env bash

CUR_IFS=$IFS

script_name="$(basename $(realpath $0))"
script_dir="$(dirname $(realpath $0))"

conf_dir="${script_dir}/conf"
snippet_dir="${script_dir}/snippets"

declare -a unsorted_website_arr
declare -a website_arr

log_file="$(mktemp)"

backup_date=$(date +%Y-%m-%d-%H%M)


# =============
# --- Some functions
# =============

clean_up() {


   if [[ -f "$_backup_crontab_file" ]]; then

      echononl "   (Re)Install previously saved crontab .."

      crontab $_backup_crontab_file >> $log_file 2>&1

      if [[ $? -eq 0 ]]; then
         echo_ok
      else
         echo_failed
         error "$(cat $log_file)"
      fi

   fi

   # Perform program exit housekeeping
   rm -f $log_file
   blank_line
   exit $1
}

is_number() {

   return $(test ! -z "${1##*[!0-9]*}" > /dev/null 2>&1);

   # - also possible
   # -
   #[[ ! -z "${1##*[!0-9]*}" ]] && return 0 || return 1
   #return $([[ ! -z "${1##*[!0-9]*}" ]])
}

echononl(){
   echo X\\c > /tmp/shprompt$$
   if [ `wc -c /tmp/shprompt$$ | awk '{print $1}'` -eq 1 ]; then
      echo -e -n "$*\\c" 1>&2
   else
       echo -e -n "$*" 1>&2
   fi
   rm /tmp/shprompt$$
}
echo_done() {
   if $terminal ; then
      echo -e "\033[75G[ \033[32mdone\033[m ]"
   else
      echo " [ done ]"
   fi
}
echo_ok() {
   if $terminal ; then
      echo -e "\033[75G[ \033[32mok\033[m ]"
   else
      echo " [ ok ]"
   fi
}
echo_warning() {
   if $terminal ; then
      echo -e "\033[75G[ \033[33m\033[1mwarn\033[m ]"
   else
      echo " [ warning ]"
   fi
}
echo_failed(){
   if $terminal ; then
      echo -e "\033[75G[ \033[1;31mfailed\033[m ]"
   else
      echo ' [ failed! ]'
   fi
}
echo_skipped() {
   if $terminal ; then
      echo -e "\033[75G[ \033[37mskipped\033[m ]"
   else
      echo " [ skipped ]"
   fi
}
fatal (){
   echo ""
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[31m\033[1mFatal\033[m ]: \033[37m\033[1m$*\033[m"
      echo ""
      echo -e "\t\033[31m\033[1m           Script will be interrupted..\033[m\033[m"
   else
      echo "fatal: $*"
      echo "Script will be interrupted.."
   fi
   clean_up 1
}

error(){
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[31m\033[1mFehler\033[m ]: $*"
   else
      echo "Error: $*"
   fi
   echo ""
}

warn (){
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[33m\033[1mWarning\033[m ]: $*"
   else
      echo "Warning: $*"
   fi
   echo ""
}

info (){
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[32m\033[1mInfo\033[m ]: $*"
   else
      echo "Info: $*"
   fi
   echo ""
}

# - Remove leading/trailling whitespaces
# -
trim() {
    local var="$*"
    var="${var#"${var%%[![:space:]]*}"}"   # remove leading whitespace characters
    var="${var%"${var##*[![:space:]]}"}"   # remove trailing whitespace characters
    echo -n "$var"
}

## - Check if a given array (parameter 2) contains a given string (parameter 1)
## -
containsElement () {
   local e
   for e in "${@:2}"; do [[ "$e" == "$1" ]] && return 0; done
   return 1
}

blank_line() {
   if $terminal ; then
      echo ""
   fi
}



# ----------
# - Jobhandling 
# ----------

# - Run 'clean_up' for signals SIGHUP SIGINT SIGTERM
# -
trap clean_up SIGHUP SIGINT SIGTERM

## - 
while IFS='' read -r -d '' _conf_file ; do
   source $_conf_file
   if [[ -n "$WEBSITE" ]] ; then
      unsorted_website_arr+=("${WEBSITE}:$_conf_file")
   fi
   WEBSITE=""
done  < <(find "${conf_dir}" -maxdepth 1 -type f -name "*.conf" -print0)

if [[ ${#unsorted_website_arr} -eq 0 ]]; then
   fatal "No configuration files found in '${script_dir}/conf' or no website configured!"
fi

# - Sort array
# -
IFS=$'\n' website_arr=($(sort <<<"${unsorted_website_arr[*]}"))


# - Reset IFS
# -
IFS=$CUR_IFS


# =============
# --- Some 
# =============

# - Support systemd ?
# -
if [[ "X$(which systemd)" = "X" ]]; then
   SYSTEMD_EXISTS=false
else
   SYSTEMD_EXISTS=true
fi

# - Running in a terminal?
# -
if [[ -t 1 ]] ; then
      terminal=true
else
   terminal=false
fi

clear

echo ""
echo -e "\033[32m-----\033[m"
echo -e "Add User to Group - \033[1mBoth, User and Group has to exist\033[m"
echo -e "\033[32m-----\033[m"


# Which cloud instance (website) would you like to update
#
source ${snippet_dir}/get-cloud-instance-to-update.sh


# - Reset IFS
# -
IFS=$CUR_IFS


echo ""
echononl "   Include Configuration file.."
if [[ ! -f $conf_file ]]; then
   echo_failed
   fatal "Missing configuration file '$conf_file'"
else
   source $conf_file
   echo_ok
fi
echo ""




# =============
# --- Some checks
# =============

DEFAULT_WEB_BASE_DIR="/var/www/$WEBSITE"
DEFAULT_HTTP_USER="www-data"
DEFAULT_HTTP_GROUP="www-data"
DEFAULT_PHP_ENGINE="FPM"


if [[ -z ${WEBSITE} ]] ; then
   fatal "No website given (parameter 'WEBSITE')"
fi

DEFAULT_WEB_BASE_DIR="/var/www/$WEBSITE"

[[ -n "$WEB_BASE_DIR" ]] || WEB_BASE_DIR=$DEFAULT_WEB_BASE_DIR

if [[ ! -d ${WEB_BASE_DIR} ]] ; then
   fatal "Web base directory not found (parameter 'WEB_BASE_DIR')"
fi

DATA_DIR="$(realpath ${WEB_BASE_DIR}/data)"

[[ -n "$PHP_ENGINE" ]] || PHP_ENGINE=$DEFAULT_PHP_ENGINE

if [[ "$DATABASE_TYPE" != "postgres" ]] && [[ "$DATABASE_TYPE" != "mysql" ]]; then
   fatal "Wrong or missing database type (parameter 'DATABASE_TYPE')"
fi

if [[ -z "$DATABASE_NAME" ]]; then
   fatal "Missing database name (parameter 'DATABASE_NAME')"
fi

if [[ "$DATABASE_TYPE" = "mysql" ]] && [[ -z "$MYSQL_CREDENTIALS" ]]; then
   fatal "No Database Credentials for MySQL given (parameter 'MYSQL_CREDENTIALS')"
fi

if [[ "$DATABASE_TYPE" = "postgres" ]]; then
   if [[ -z "$PSQL_USER" ]] || [[ -z "$PSQL_PASS" ]]; then
      fatal "No Database Credentials for PostgreSQL given (parameters: 'PSQL_USER' 'PSQL_PASS'"
   fi
fi


NGINX_IS_ENABLED=false
APACHE2_IS_ENABLED=false

# Get Webservice environment as IS_HTTPD_RUNNING, HTTP_USER, HTTP_GROUP..
#
source ${snippet_dir}/get-webservice-environment.sh


# Check PHP Version
#
source ${snippet_dir}/get-php-major-version.sh


# Get full qualified PHP command
#
source ${snippet_dir}/get-path-of-php-command.sh


if [[ ! -x "$PHP_BIN" ]]; then
   fatal "No PHP binary found!"
fi

CURRENT_INSTALL_DIR=`realpath ${WEB_BASE_DIR}/nextcloud`
CURRENT_VERSION=`basename $CURRENT_INSTALL_DIR | cut -d"-" -f2`

echo ""
echo -e "\033[1;32mStarting Script $script_name for \033[1;37m${WEBSITE}\033[m"
echo ""
echo -e   "   Cloud instance to be changed.........: $WEBSITE"
echo ""
echo -e   "   Current version of nextcloud.........: $CURRENT_VERSION"
echo ""
echo ""
echo ""
echo -e   "   Web base directory...................: $WEB_BASE_DIR"
echo ""
echo -e   "   Webserver user.......................: $HTTP_USER"
echo -e   "   Webserver group......................: $HTTP_GROUP"
echo ""
echo -e   "   PHP Command..........................: $PHP_BIN"
echo ""
echo -e   "   PHP version..........................: $PHP_VERSION"
echo -e   "   PHP Engine...........................: $PHP_ENGINE"
echo ""
echo -e   "   Databse name.........................: $DATABASE_NAME"
echo -e   "   Database type........................: $DATABASE_TYPE"
echo ""
if [[ "$DATABASE_TYPE" = "mysql" ]] ; then
   echo -e   "   Mysql Credentials....................: $MYSQL_CREDENTIALS"
fi
echo ""

echo ""
echo -n "   Type upper case 'YES' to continue executing with this parameters: "
read OK
if [[ "$OK" = "YES" ]] ; then
   echo ""
   echo ""
   echo -e "\033[1;32mGoing toeck for old files on \033[1;37m$WEBSITE \033[m"
   echo ""
else
   fatal "Abort by user request - Answer as not 'YES'"
fi


echo ""


# -----
# - Main part of the script
# -----

echo ""
echo ""
echo -e "\033[37m\033[1mMain part of the script\033[m"
echo ""


# - Checking for old files
# -
echononl "   Check, if we can disable legacy encryption - Checking for old files.."

su -c "${PHP_BIN} ${WEB_BASE_DIR}/htdocs/occ encryption:scan:legacy-format" -s /bin/bash $HTTP_USER > $log_file 2>&1
if [[ $? -eq 0 ]]; then
   echo_ok
   info "Output of command \033[37m\033[1mocc encryption:scan:legacy-format\033[m was:\n\n$(cat $log_file)"
else
   echo_failed
   blank_line
   echo ""
   echo -e "\033[37m\033[1mcommandline was:\033[m"
   echo "su -c \"${PHP_BIN} ${WEB_BASE_DIR}/htdocs/occ encryption:scan:legacy-format\" -s /bin/bash $HTTP_USER"
   error "Output of command \033[37m\033[1mocc encryption:scan:legacy-format\033[m was:\n\n$(cat $log_file)"
fi

blank_line

clean_up 0
