#!/usr/bin/env bash

CUR_IFS=$IFS

script_name="$(basename $(realpath $0))"
script_dir="$(dirname $(realpath $0))"

conf_dir="${script_dir}/conf"
snippet_dir="${script_dir}/snippets"

declare -a unsorted_website_arr
declare -a website_arr

log_file="$(mktemp)"

backup_date=$(date +%Y-%m-%d-%H%M)


# =============
# --- Some functions
# =============

clean_up() {


   if [[ -f "$_backup_crontab_file" ]]; then

      echononl "   (Re)Install previously saved crontab .."

      crontab $_backup_crontab_file >> $log_file 2>&1

      if [[ $? -eq 0 ]]; then
         echo_ok
      else
         echo_failed
			error "$(cat $log_file)"
      fi

   fi

   # Perform program exit housekeeping
   rm -f $log_file
   blank_line
   exit $1
}

is_number() {

   return $(test ! -z "${1##*[!0-9]*}" > /dev/null 2>&1);

   # - also possible
   # -
   #[[ ! -z "${1##*[!0-9]*}" ]] && return 0 || return 1
   #return $([[ ! -z "${1##*[!0-9]*}" ]])
}

echononl(){
   echo X\\c > /tmp/shprompt$$
   if [ `wc -c /tmp/shprompt$$ | awk '{print $1}'` -eq 1 ]; then
      echo -e -n "$*\\c" 1>&2
   else
       echo -e -n "$*" 1>&2
   fi
   rm /tmp/shprompt$$
}
echo_done() {
   if $terminal ; then
      echo -e "\033[75G[ \033[32mdone\033[m ]"
   else
      echo " [ done ]"
   fi
}
echo_ok() {
   if $terminal ; then
      echo -e "\033[75G[ \033[32mok\033[m ]"
   else
      echo " [ ok ]"
   fi
}
echo_warning() {
   if $terminal ; then
      echo -e "\033[75G[ \033[33m\033[1mwarn\033[m ]"
   else
      echo " [ warning ]"
   fi
}
echo_failed(){
   if $terminal ; then
      echo -e "\033[75G[ \033[1;31mfailed\033[m ]"
   else
      echo ' [ failed! ]'
   fi
}
echo_skipped() {
   if $terminal ; then
      echo -e "\033[75G[ \033[37mskipped\033[m ]"
   else
      echo " [ skipped ]"
   fi
}
fatal (){
   echo ""
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[31m\033[1mFatal\033[m ]: \033[37m\033[1m$*\033[m"
      echo ""
      echo -e "\t\033[31m\033[1m           Script will be interrupted..\033[m\033[m"
   else
      echo "fatal: $*"
      echo "Script will be interrupted.."
   fi
   clean_up 1
}

error(){
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[31m\033[1mFehler\033[m ]: $*"
   else
      echo "Error: $*"
   fi
   echo ""
}

warn (){
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[33m\033[1mWarning\033[m ]: $*"
   else
      echo "Warning: $*"
   fi
   echo ""
}

info (){
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[32m\033[1mInfo\033[m ]: $*"
   else
      echo "Info: $*"
   fi
   echo ""
}

## - Check if a given array (parameter 2) contains a given string (parameter 1)
## -
containsElement () {
   local e
   for e in "${@:2}"; do [[ "$e" == "$1" ]] && return 0; done
   return 1
}

blank_line() {
   if $terminal ; then
      echo ""
   fi
}



# ----------
# - Jobhandling 
# ----------

# - Run 'clean_up' for signals SIGHUP SIGINT SIGTERM
# -
trap clean_up SIGHUP SIGINT SIGTERM

## - 
while IFS='' read -r -d '' _conf_file ; do
   source $_conf_file
   if [[ -n "$WEBSITE" ]] ; then
      unsorted_website_arr+=("${WEBSITE}:$_conf_file")
   fi
   WEBSITE=""
done  < <(find "${conf_dir}" -maxdepth 1 -type f -name "*.conf" -print0)
#done  < <(find "${conf_dir}" -maxdepth 1 -type f -name "create_missing_indices_*.conf" -print0)

#if [[ ${#unsorted_website_arr} -eq 0 ]]; then
#   fatal "No configuration files found in '${script_dir}/conf' or no website configured!"
#fi

# - Sort array
# -
IFS=$'\n' website_arr=($(sort <<<"${unsorted_website_arr[*]}"))


# - Reset IFS
# -
IFS=$CUR_IFS


# =============
# --- Some 
# =============

# - Support systemd ?
# -
if [[ "X$(which systemd)" = "X" ]]; then
   SYSTEMD_EXISTS=false
else
   SYSTEMD_EXISTS=true
fi

# - Running in a terminal?
# -
if [[ -t 1 ]] ; then
      terminal=true
else
   terminal=false
fi

clear

echo ""
echo -e "\033[32m-----\033[m"
echo "Add missing columns"
echo -e "\033[32m-----\033[m"

# Which cloud instance (website) would you like to update
#
source ${snippet_dir}/get-cloud-instance-to-update.sh


# - Reset IFS
# -
IFS=$CUR_IFS


echo ""
echononl "   Include Configuration file.."
if [[ ! -f $conf_file ]]; then
   echo_failed
   fatal "Missing configuration file '$conf_file'."
else
   source $conf_file
   echo_ok
fi
echo ""



# =============
# --- Some checks
# =============

DEFAULT_SRC_BASE_DIR="/usr/local/src/nextcloud"
DEFAULT_HTTP_USER="www-data"
DEFAULT_HTTP_GROUP="www-data"
DEFAULT_PHP_ENGINE="FPM"


if [[ -z ${WEBSITE} ]] ; then
   WEBSITE="$DEFAULT_WEBSITE"
fi

DEFAULT_WEB_BASE_DIR="/var/www/$WEBSITE"

[[ -n "$WEB_BASE_DIR" ]] || WEB_BASE_DIR=$DEFAULT_WEB_BASE_DIR

if [[ ! -d ${WEB_BASE_DIR} ]] ; then
   fatal "Web base directory not found (parameter 'WEB_BASE_DIR')"
fi

[[ -n "$PHP_ENGINE" ]] || PHP_ENGINE=$DEFAULT_PHP_ENGINE

if [[ "$DATABASE_TYPE" != "postgres" ]] && [[ "$DATABASE_TYPE" != "mysql" ]]; then
   fatal "Wrong or missing database type (parameter 'DATABASE_TYPE')"
fi

if [[ -z "$DATABASE_NAME" ]]; then
   fatal "Missing database name (parameter 'DATABASE_NAME')"
fi

if [[ "$DATABASE_TYPE" = "mysql" ]] && [[ -z "$MYSQL_CREDENTIALS" ]]; then
   fatal "No Database Credentials for MySQL given (parameter 'MYSQL_CREDENTIALS')"
fi

if [[ "$DATABASE_TYPE" = "postgres" ]]; then
   if [[ -z "$PSQL_USER" ]] || [[ -z "$PSQL_PASS" ]]; then
      fatal "No Database Credentials for PostgreSQL given (parameters: 'PSQL_USER' 'PSQL_PASS'"
   fi
fi

NGINX_IS_ENABLED=false
APACHE2_IS_ENABLED=false

# Get Webservice environment as IS_HTTPD_RUNNING, HTTP_USER, HTTP_GROUP..
#
source ${snippet_dir}/get-webservice-environment.sh

# Check PHP Version
#
source ${snippet_dir}/get-php-major-version.sh


# Get full qualified PHP command
#
source ${snippet_dir}/get-path-of-php-command.sh


CURRENT_INSTALL_DIR="$(realpath ${WEB_BASE_DIR}/nextcloud)"
CURRENT_VERSION="$(basename $CURRENT_INSTALL_DIR | cut -d"-" -f2)"

echo ""
echo -e "\033[1;32mStarting Script $script_name for \033[1;37m${WEBSITE}\033[m"
echo ""
echo -e   "   Website to update....................: $WEBSITE"
echo ""
echo -e   "   Nextcloud version....................: $CURRENT_VERSION"
echo ""
echo -e   "   Web base directory...................: $WEB_BASE_DIR"
echo ""
echo -e   "   Webserver user.......................: $HTTP_USER"
echo -e   "   Webserver group......................: $HTTP_GROUP"
echo ""
echo -e   "   PHP version..........................: $PHP_VERSION"
echo -e   "   PHP Engine...........................: $PHP_ENGINE"
echo ""
echo -e   "   PHP Command..........................: $PHP_BIN"

echo ""
echo -n "   Type upper case 'YES' to continue executing with this parameters: "
read OK
if [[ "$OK" = "YES" ]] ; then
   echo ""
   echo ""
   echo -e "\033[1;32mGoing to update Nextcloud on \033[1;37m$WEBSITE \033[m"
   echo ""
else
   fatal "Abort by user request - Answer as not 'YES'"
fi


echo ""


# -----
# - Do some pre-update tasks..
# -----

echo ""
echo ""
echo -e "\033[37m\033[1mDo some pre-update tasks..\033[m"
echo ""

# - Deaktiviere Cronjobs
# -
_backup_crontab_file=/tmp/crontab_root.${backup_date}
echononl "   Backup Crontab to '$_backup_crontab_file'"
crontab -l > $_backup_crontab_file 2> $log_file
if [[ $? -eq 0 ]]; then
   echo_ok
else
   echo_failed
   fatal "$(cat $log_file)"
fi

echononl "   Remove crontab for root.."
crontab -r > $log_file 2>&1
if [[ $? -eq 0 ]]; then
   echo_ok
else
   echo_failed
   fatal "$(cat $log_file)"
fi


# - Stop Apache Webserver
# -
echo ""
echononl "   Stop Apache Webserver.."
if $APACHE2_IS_ENABLED ; then
   if $IS_HTTPD_RUNNING ; then
      if $SYSTEMD_EXISTS ; then
         systemctl stop apache2
         if [[ $? -eq 0 ]]; then
            echo_ok
         else
            echo_failed
            error "$(cat $log_file)"

            echononl "continue anyway [yes/no]: "
            read OK
            OK="$(echo "$OK" | tr '[:upper:]' '[:lower:]')"
            while [[ "$OK" != "yes" ]] && [[  "$OK" != "no" ]] ; do
               echononl "Wrong entry! - repeat [yes/no]: "
               read OK
            done
            [[ $OK = "yes" ]] || fatal "Interrupted by user."

         fi
      else
         /etc/init.d/apache2 stop
         if [[ $? -eq 0 ]]; then
            echo_ok
         else
            echo_failed
            error "$(cat $log_file)"

            echononl "continue anyway [yes/no]: "
            read OK
            OK="$(echo "$OK" | tr '[:upper:]' '[:lower:]')"
            while [[ "$OK" != "yes" ]] && [[  "$OK" != "no" ]] ; do
               echononl "Wrong entry! - repeat [yes/no]: "
               read OK
            done
            [[ $OK = "yes" ]] || fatal "Interrupted by user."

         fi
      fi
   else
      echo_skipped
   fi
else
   echo_skipped
fi

# - Stop Nginx Webservice
# -
echo ""
echononl "   Stop Nginx Webserver.."
if $NGINX_IS_ENABLED ; then
   if $IS_HTTPD_RUNNING ; then
      if $SYSTEMD_EXISTS ; then
         systemctl stop nginx
         if [[ $? -eq 0 ]]; then
            echo_ok
         else
            echo_failed
            error "$(cat $log_file)"

            echononl "continue anyway [yes/no]: "
            read OK
            OK="$(echo "$OK" | tr '[:upper:]' '[:lower:]')"
            while [[ "$OK" != "yes" ]] && [[  "$OK" != "no" ]] ; do
               echononl "Wrong entry! - repeat [yes/no]: "
               read OK
            done
            [[ $OK = "yes" ]] || fatal "Interrupted by user."

         fi
      else
         /etc/init.d/nginx stop
         if [[ $? -eq 0 ]]; then
            echo_ok
         else
            echo_failed
            error "$(cat $log_file)"

            echononl "continue anyway [yes/no]: "
            read OK
            OK="$(echo "$OK" | tr '[:upper:]' '[:lower:]')"
            while [[ "$OK" != "yes" ]] && [[  "$OK" != "no" ]] ; do
               echononl "Wrong entry! - repeat [yes/no]: "
               read OK
            done
            [[ $OK = "yes" ]] || fatal "Interrupted by user."

         fi
      fi
   else
      echo_skipped
   fi
fi

echo ""

# - Backup Database
# -
echononl "   Backup MySQL database '$DATABASE_NAME'.."
if [[ "$DATABASE_TYPE" = 'mysql' ]]; then
   mysqldump $MYSQL_CREDENTIALS --max_allowed_packet=128M --single-transaction $DATABASE_NAME > \
      ${WEB_BASE_DIR}/${DATABASE_NAME}-v${CURRENT_VERSION}.${backup_date}.sql 2> $log_file
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed

      blank_line
      echo -e "\t[ \033[33m\033[1mCommand\033[m ]: \033[37m\033[1mmysqldump $MYSQL_CREDENTIALS \\
\t                --max_allowed_packet=128M --single-transaction $DATABASE_NAME > \\
\t                ${WEB_BASE_DIR}/${DATABASE_NAME}-v${PRIOR_VERSION}.${backup_date}.sql\033[m"

      error "$(cat $log_file)"

      echononl "continue anyway [yes/no]: "
      read OK
      OK="$(echo "$OK" | tr '[:upper:]' '[:lower:]')"
      while [[ "$OK" != "yes" ]] && [[  "$OK" != "no" ]] ; do
         echononl "Wrong entry! - repeat [yes/no]: "
         read OK
      done
      [[ $OK = "yes" ]] || fatal "Abbruch durch User"
      blank_line

   fi
elif [[ "$DATABASE_TYPE" = 'postgres' ]]; then
   PGPASSWORD=$PSQL_PASS \
   pg_dump $DATABASE_NAME -h $PSQL_SERVER -U $PSQL_USER  \
      -f ${WEB_BASE_DIR}/${DATABASE_NAME}-v${CURRENT_VERSION}.${backup_date}.sql 2> $log_file
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      fatal "$(cat $log_file)"
   fi
fi

echo


# -----
# - Main part of the script
# -----

echo ""
echo ""
echo -e "\033[37m\033[1mMain part of the script\033[m"
echo ""


# - Add missing columns
# -
echononl "   Migrating the mimetypes .."
su -c "${PHP_BIN} ${WEB_BASE_DIR}/htdocs/occ maintenance:repair --include-expensive" -s /bin/bash $HTTP_USER > $log_file 2>&1
if [[ $? -eq 0 ]]; then
   echo_ok
else
   echo_failed
   fatal "$(cat $log_file)"
fi




# -----
# - Doing some post-update tasks
# -----

echo ""
echo ""
echo -e "\033[37m\033[1mDoing some post-update tasks..\033[m"

echo ""
echononl "   Restart PHP engine.."
if [[ "$PHP_ENGINE" = "FPM" ]]; then
   if $SYSTEMD_EXISTS ; then
      systemctl restart php-${PHP_VERSION}-fpm > $log_file 2>&1
      if [[ $? -eq 0 ]]; then
         echo_ok
      else
         echo_failed
			error "$(cat $log_file)"

         echononl "continue anyway [yes/no]: "
         read OK
         OK="$(echo "$OK" | tr '[:upper:]' '[:lower:]')"
         while [[ "$OK" != "yes" ]] && [[  "$OK" != "no" ]] ; do
            echononl "Wrong entry! - repeat [yes/no]: "
            read OK
         done
   		[[ $OK = "yes" ]] || fatal "Interrupted by user."
      fi
   else
      /etc/init.d/php-${PHP_VERSION}-fpm restart > $log_file 2>&1
      if [[ $? -eq 0 ]]; then
         echo_ok
      else
         echo_failed
			error "$(cat $log_file)"

         echononl "continue anyway [yes/no]: "
         read OK
         OK="$(echo "$OK" | tr '[:upper:]' '[:lower:]')"
         while [[ "$OK" != "yes" ]] && [[  "$OK" != "no" ]] ; do
            echononl "Wrong entry! - repeat [yes/no]: "
            read OK
         done
   		[[ $OK = "yes" ]] || fatal "Interrupted by user."
      fi
   fi
else
   echo_skipped
fi

# - Start Apache Webserver
# -
echo ""
echononl "   Start Apache Webserver.."
if $APACHE2_IS_ENABLED ; then
   if $IS_HTTPD_RUNNING ; then
      if $SYSTEMD_EXISTS ; then
         systemctl start apache2 > $log_file 2>&1
         if [[ $? -eq 0 ]]; then
            echo_ok
         else
            echo_failed
            error "$(cat $log_file)"

            echononl "continue anyway [yes/no]: "
            read OK
            OK="$(echo "$OK" | tr '[:upper:]' '[:lower:]')"
            while [[ "$OK" != "yes" ]] && [[  "$OK" != "no" ]] ; do
               echononl "Wrong entry! - repeat [yes/no]: "
               read OK
            done
            [[ $OK = "yes" ]] || fatal "Interrupted by user."

         fi
      else
         /etc/init.d/apache2 start > $log_file 2>&1
         if [[ $? -eq 0 ]]; then
            echo_ok
         else
            echo_failed
            error "$(cat $log_file)"

            echononl "continue anyway [yes/no]: "
            read OK
            OK="$(echo "$OK" | tr '[:upper:]' '[:lower:]')"
            while [[ "$OK" != "yes" ]] && [[  "$OK" != "no" ]] ; do
               echononl "Wrong entry! - repeat [yes/no]: "
               read OK
            done
            [[ $OK = "yes" ]] || fatal "Interrupted by user."
         fi
      fi
   else
      echo_skipped
      warn "The webserver was not running, so it will be keept down!"
   fi
else
   echo_skipped
fi


# - Start NGINX Webservise
# -
echo ""
echononl "   Start Nginx Webserver.."
if $NGINX_IS_ENABLED ; then
   if $IS_HTTPD_RUNNING ; then
      if $SYSTEMD_EXISTS ; then
         systemctl start nginx > $log_file 2>&1
         if [[ $? -eq 0 ]]; then
            echo_ok
         else
            echo_failed
            error "$(cat $log_file)"

            echononl "continue anyway [yes/no]: "
            read OK
            OK="$(echo "$OK" | tr '[:upper:]' '[:lower:]')"
            while [[ "$OK" != "yes" ]] && [[  "$OK" != "no" ]] ; do
               echononl "Wrong entry! - repeat [yes/no]: "
               read OK
            done
            [[ $OK = "yes" ]] || fatal "Interrupted by user."

         fi
      else
         /etc/init.d/nginx start > $log_file 2>&1
         if [[ $? -eq 0 ]]; then
            echo_ok
         else
            echo_failed
            error "$(cat $log_file)"

            echononl "continue anyway [yes/no]: "
            read OK
            OK="$(echo "$OK" | tr '[:upper:]' '[:lower:]')"
            while [[ "$OK" != "yes" ]] && [[  "$OK" != "no" ]] ; do
               echononl "Wrong entry! - repeat [yes/no]: "
               read OK
            done
            [[ $OK = "yes" ]] || fatal "Interrupted by user."
         fi
      fi
   else
      echo_skipped
      warn "The NGINX is not configured as active - so nothing to do."
   fi
else
   echo_skipped
fi

_redis_cli_bin="$(which redis-cli)"
if [[ -z "$_redis_cli_bin" ]]; then
   if [[ -x "/usr/local/bin/redis-cli" ]]; then
      _redis_cli_bin="/usr/local/bin/redis-cli"
   fi
fi


echononl "   Flush redis cache.."
if [[ -x "$_redis_cli_bin" ]]; then
   $_redis_cli_bin flushall > $log_file 2>&1
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      error "$(cat $log_file)"
   fi
else
   echo_skipped
fi


echononl "   Restart redis server.."
if $SYSTEMD_EXISTS ; then
   systemctl restart redis-server > $log_file 2>&1
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      error "$(cat $log_file)"
   fi
else
   /etc/init.d/redis-server restart  > $log_file 2>&1
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      error "$(cat $log_file)"
   fi
fi

blank_line
clean_up 0
